package envoy.server;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;

import com.jenkov.nioserver.IMessageReader;
import com.jenkov.nioserver.Message;
import com.jenkov.nioserver.MessageBuffer;
import com.jenkov.nioserver.Socket;

/**
 * This {@link IMessageReader} decodes serialized Java objects.<br>
 * <br>
 * Project: <strong>envoy-server-standalone</strong><br>
 * File: <strong>ObjectMessageReader.java</strong><br>
 * Created: <strong>28.12.2019</strong><br>
 *
 * @author Kai S. K. Engelbart
 * @since Envoy Server Standalone v0.1-alpha
 */
public class ObjectMessageReader implements IMessageReader {

	private List<Message>	completeMessages	= new ArrayList<>();
	private Message			nextMessage;
	private MessageBuffer	messageBuffer;

	private int fromByteArray(byte[] bytes, int offset) {
		return ((bytes[offset] & 0xFF) << 24) | ((bytes[offset + 1] & 0xFF) << 16) | ((bytes[offset + 2] & 0xFF) << 8)
				| ((bytes[offset + 3] & 0xFF) << 0);
	}

	@Override
	public List<Message> getMessages() { return completeMessages; }

	@Override
	public void init(MessageBuffer messageBuffer) {
		this.messageBuffer	= messageBuffer;
		nextMessage			= messageBuffer.getMessage();
	}

	@Override
	public void read(Socket socket, ByteBuffer buffer) throws IOException {
		socket.read(buffer);
		buffer.flip();

		if (!buffer.hasRemaining()) {
			buffer.clear();
			return;
		}

		nextMessage.writeToMessage(buffer);

		// Get message length
		if (nextMessage.length - nextMessage.offset < 4) return;
		int length = fromByteArray(nextMessage.sharedArray, nextMessage.offset) + 4;

		if (nextMessage.length - nextMessage.offset >= length) {
			Message message = messageBuffer.getMessage();
			message.writePartialMessageToMessage(nextMessage, nextMessage.offset + length);
			completeMessages.add(nextMessage);
			nextMessage = message;
		}

		buffer.clear();
	}
}